/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: denora.h 437 2005-07-27 16:45:53Z drstein $
 *
 */

#ifndef STATS_H
#define STATS_H

/*************************************************************************/

#include "sysconf.h"

/* Some SUN fixs */
#ifdef __sun
/* Solaris specific code, types that do not exist in Solaris'
 * sys/types.h
 **/
#undef u_int8_t
#undef u_int16_t
#undef u_int32_t
#undef u_int_64_t
#define u_int8_t uint8_t
#define u_int16_t uint16_t
#define u_int32_t uint32_t
#define u_int64_t uint64_t

#ifndef INADDR_NONE
#define INADDR_NONE (-1)
#endif

#endif

#include <stddef.h>
#include <stdarg.h>
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifndef _WIN32
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#endif
#include <signal.h>
#include <time.h>
#include <errno.h>
#ifndef _WIN32
#include <grp.h>
#endif
#include <limits.h>
#ifndef _WIN32
#include <netdb.h>
#ifdef HAVE_NETINETIN
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <sys/socket.h>
#else
#include <winsock.h>
#include <windows.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>	/* for umask() on some systems */
#ifndef _WIN32
#include <sys/time.h>
#endif
#ifdef _WIN32
#include <sys/timeb.h>
#include <direct.h>
#include <io.h>
#endif

#include <fcntl.h>

#ifndef _WIN32
#include <sys/param.h>
#include <sys/dir.h>
#endif
#include <assert.h>

#ifdef HAVE_STDINT_H
#include <stdint.h>     /* For uint32_t */
#endif

#ifdef HAVE_LIBZ
#include <zlib.h>
#endif

#ifndef _WIN32
#ifdef HAVE_BACKTRACE
#include <execinfo.h>
#endif
#endif

#ifdef USE_MYSQL
#ifdef MYSQL_HEADER_PREFIX
# include <mysql/mysql.h>
# include <mysql/errmsg.h>
# include <mysql/mysqld_error.h> 
#else
# include <mysql.h>
# include <errmsg.h>
# include <mysqld_error.h> 
#endif
#endif

#ifdef USE_POSTGRE
#include <libpq-fe.h>
#endif

#if HAVE_STRINGS_H
# include <strings.h>
#endif

#if HAVE_SYS_SELECT_H
# include <sys/select.h>
#endif

#include "defines.h"

#ifdef _AIX
/* Some AIX boxes seem to have bogus includes that don't have these
 * prototypes. */
extern int strcasecmp(const char *, const char *);
extern int strncasecmp(const char *, const char *, size_t);
# if 0  /* These break on some AIX boxes (4.3.1 reported). */
extern int gettimeofday(struct timeval *, struct timezone *);
extern int socket(int, int, int);
extern int bind(int, struct sockaddr *, int);
extern int connect(int, struct sockaddr *, int);
extern int shutdown(int, int);
# endif
# undef FD_ZERO
# define FD_ZERO(p) memset((p), 0, sizeof(*(p)))
#undef int16
#undef int32
# define int16 builtin_int16
# define int32 builtin_int32
#endif /* _AIX */

/* Alias stricmp/strnicmp to strcasecmp/strncasecmp if we have the latter
 * but not the former. */
#ifndef HAVE_STRICMP 
#ifdef HAVE_STRCASECMP
# define stricmp strcasecmp
# define strnicmp strncasecmp
#endif
#endif

/* We have our own versions of toupper()/tolower(). */
#include <ctype.h>
#undef tolower
#undef toupper
#define tolower tolower_
#define toupper toupper_

#ifdef __WINS__
#ifndef BKCHECK
#define BKCHECK
  extern "C" void __pfnBkCheck() {}
#endif
#endif

/* Length of an array: */
#define lenof(a)	(sizeof(a) / sizeof(*(a)))

/* Telling compilers about printf()-like functions: */
#ifdef __GNUC__
# define FORMAT(type,fmt,start) __attribute__((format(type,fmt,start)))
#else
# define FORMAT(type,fmt,start)
#endif

/* Misc macros */
#define IsDigit(c) (char_atribs[(unsigned char)(c)]&DIGIT)
#define BadPtr(x) (!(x) || (*(x) == '\0'))

#include "GeoIP.h"
#include "zip.h"
#include "events.h"
#include "sockets.h"
#include "version.h"
#include "ircd.h"
#include "modes.h"
#include "cron.h"

/*************************************************************************/

typedef struct server_ Server;
typedef struct user_ User;
typedef struct channel_ Channel;
typedef struct ModuleData_ ModuleData;              /* ModuleData struct */
typedef struct bandata_ BanData;
typedef struct userdata_ UserData;
typedef struct sxline_ SXLine;
typedef struct cbmode_ CBMode;
typedef struct cbmodeinfo_ CBModeInfo;
typedef struct cmmode_ CMMode;
typedef struct csmode_ CSMode;
typedef struct cumode_ CUMode;
typedef struct tld_ TLD;
typedef struct ircdvars_ IRCDVar;
typedef struct statvars_ STATVar;
typedef struct ircdcapab_ IRCDCAPAB;
typedef struct denoravars_ DenoraVar;
typedef struct uid_ Uid;
typedef struct serverstats_ ServStats;
typedef struct ctcpverstats_ CTCPVerStats;
typedef struct exclude_ Exclude;
typedef struct chanstats_ ChannelStats;
typedef struct statschan_ StatsChannel;
typedef struct db_file_ DenoraDBFile;

/* Configuration */
typedef struct Conf_Admin aAdmin;
typedef struct Conf_Modules Conf_Modules;

/* tiny struct needed for P10 and other UID servers so we can track 
   services UID
*/
struct uid_ {
    Uid *next, *prev;
    char nick[NICKMAX];
    char *uid;
};

/*************************************************************************/

/* Windows defines a boolean type as an 
 * unsigned char. It does however need
 * true/false. -- codemastr
 */
#ifndef _WIN32
typedef enum { false, true } boolean;
#else
#ifndef true
#define true 1
#endif
#ifndef false
#define false 0
#endif
#endif /* _WIN32 */


/*************************************************************************/

struct db_file_ {
    FILE *fptr;             /* Pointer to the opened file */
    int db_version;         /* The db version of the datafiles (only needed for reading) */
    int core_db_version;    /* The current db version of this anope source */
    char service[256];      /* StatServ/etc. */
    char filename[256];     /* Filename of the database */
    char temp_name[262];    /* Temp filename of the database */
};

/*************************************************************************/

struct denoravars_ {
	const char *dir;
	const char *logname;
	const char *config;
	const char *version;
	const char *versiondotted;
	const char *flags_os;
	const char *flag_mods;
	const char *flag_sql;
	char *version_protocol;
	const char *date;
	const char *time;
	const char *build;
	char *qmsg;
	time_t start_time;
	int do_sql;
	int sqldebug;
	char *uplink;
	uint32 capab;
	int quitting;
	int delayed_quit;
	int do_html;
	int socketdebug;
	int debug;
	int nofork;
	int save_data;
	int protocoldebug;
};

struct statvars_ {
	uint32 users;
	uint32 opers;
	uint32 chans;
	uint32 servers;
	uint32 away;

	uint32 users_max;
	time_t users_max_time;
	uint32 opers_max;
	time_t opers_max_time;
	uint32 chans_max;
	time_t chans_max_time;
	uint32 servers_max;
	time_t servers_max_time;

	uint32 daily_users;
	time_t daily_users_time;
	uint32 daily_opers;
	time_t daily_opers_time;
	uint32 daily_servers;
	time_t daily_servers_time;
	uint32 daily_chans;
	time_t daily_chans_time;

	uint32 totalusersever;

};

/* Protocol tweaks */


struct ircdvars_ {
	const char *name;				/* Name of the ChanServ command */
	const char *operservmode;		/* Mode used by OperServ	*/
	int vhost;				/* Supports vhost		*/
	int sgline;				/* Supports SGline		*/
	int sgline_table;       /* Whether we need the sgline table */
	int sqline;				/* Supports SQline		*/
	int sqline_table;       /* Whether we need the sqline table */
	int szline;				/* Supports SZline		*/
	int except;				/* exception +e			*/
	int vident;				/* Supports vidents		*/
	int nickip;				/* Sends IP on NICK		*/
	int nickvhost;				/* Users vhost sent during NICK */
	int fmode;					/* +f				*/
	int Lmode;					/* +L				*/
	uint32 chan_fmode;			/* Mode 			*/
	uint32 chan_lmode;			/* Mode				*/
	char *chanmodes;			/* If the ircd sends CHANMODE in CAPAB this is where we store it */
	int token;				/* Does Denora support the tokens for the ircd */
	int tokencaseless;			/* TOKEN are not case senstive - most its Unreal that is case senstive */
	int sjb64;				/* Base 64 encode TIMESTAMP */
    int invitemode;				/* +I  */
    int sjoinbanchar;			/* use single quotes to define it */
    int sjoinexchar;			/* use single quotes to define it */
	uint32 vhostmode;			/* vhost mode */
	int owner;
	int protect;
	int halfop;
	const char *usermodes;
	const char *cmodes;
    int floodchar;
    int floodchar_alternative;
	int vhostchar;
	int vhostchar2;
	int p10;
	int ts6;
	int numerics;			/* IRCD uses Numerics to respond to some messages */
	int gagged;				/* channel mode where user can not speak but is not banned */
	int spamfilter;
	int ban_char;
	int except_char;
	int invite_char;
	int zip;
	int uline;
	char *nickchars;
	int has_svid;
};

/*************************************************************************/

struct ircdcapab_ {
  uint32 noquit;
  uint32 tsmode;
  uint32 unconnect;  
  uint32 nickip;
  uint32 nsjoin;
  uint32 zip;
  uint32 burst;
  uint32 ts5;
  uint32 ts3;
  uint32 dkey;
  uint32 pt4;
  uint32 scs;
  uint32 qs;
  uint32 uid;
  uint32 knock;
  uint32 client;
  uint32 ipv6;
  uint32 ssj5;
  uint32 sn2;
  uint32 token;
  uint32 vhost;
  uint32 ssj3;
  uint32 nick2;
  uint32 umode2;
  uint32 vl;
  uint32 tlkext;
  uint32 dodkey;
  uint32 dozip;
  uint32 chanmodes;
  uint32 sjb64;
  uint32 nickchars;
};

/*************************************************************************/

/**
 * ModuleData strucs used to allow modules to add / delete module Data from existing structs
 */

struct ModuleData_ {
	char *moduleName;						/* Which module we belong to */
	char *key;								/* The key */
	char *value;							/* The Value */
	ModuleData *next;						/* The next ModuleData record */
};
 
/*************************************************************************/

/* Server data */

struct server_ {
    Server *next, *prev;
    
    char name[NICKMAX];	/* Server name 							*/
    uint16 hops;		/* Hops between services and server 	*/
    char *desc;			/* Server description 					*/
    uint16 flags;		/* Some info flags					 	*/
    char *suid;			/* Server Universal ID		     		*/
    int sync;	        /* Whether is synced or not	        	*/
    char *version;		/* Version 								*/
    uint32 uptime;		/* Uptime								*/		
    ServStats *ss;
    char *uplink;		/* Uplink Server Name					*/
    time_t lag_time;
    uint32 ping;
    uint32 lastping;
    uint32 sqlid;		/* server sql id */
    Server *links;		/* Linked list head for linked servers 	  */
    Server *uplinks;    /* Server which pretends to be the uplink */
    int uline;
    ModuleData *moduleData;
};

struct serverstats_ {
	ServStats *next, *prev;

    char name[NICKMAX];		/* Server name 							*/
    uint32 uptime;			/* Uptime								*/
	uint32 currentusers; 	/* Curretn Users 						*/
	uint32 maxusers;		/* Max Users							*/
	uint32 opers;			/* Current Opers						*/
	uint32 maxopers;		/* Max Opers							*/
	time_t maxopertime;		/* Time of Max Opers					*/
	time_t maxusertime;		/* Time of Max Users					*/
	uint32 highestping;     /* Largest Ping 						*/
	time_t maxpingtime;		/* Largest ping time					*/
	uint32 totalusersever;  /* Total users ever						*/
	uint32 splits;			/* Total number of server splits		*/
	uint32 ircopskills;		
	uint32 serverkills;
	uint32 split_stats;
	time_t lastseen;
	ModuleData *moduleData;
};

/*************************************************************************/

struct tld_ {
    TLD *next, *prev;
    TLD *nextsort, *prevsort;

    char *country;
    char countrycode[NICKMAX];
    uint32 count;
    uint32 overall;
    ModuleData *moduleData;
};

/*************************************************************************/

struct ctcpverstats_ {
    CTCPVerStats *next, *prev;
    CTCPVerStats *nextsort, *prevsort;

    char *version;
    uint32 count;
    uint32 overall;
    ModuleData *moduleData;
};

/*************************************************************************/

struct exclude_ {
    Exclude *next, *prev;

    char *name;
    uint32 flag;
};

/*************************************************************************/

/* Online user and channel data. */
struct user_ {
    User *next, *prev;

    char nick[NICKMAX];

    char *username;		/* ident			*/
    char *host;         /* User's real hostname 	*/
    char *vhost;        /* User's virtual hostname 	*/
    char *vident;       /* User's virtual ident 	*/
    char *realname;		/* Realname 			*/
    Server *server;		/* Server user is connected to  */
    char *ip;			/* NICK IP */
    time_t timestamp;	/* Timestamp of the nick 	*/
    time_t my_signon;	/* When did _we_ see the user?  */
    uint32 svid;		/* Services ID 			*/
    uint32 mode;		/* See below 			*/
    char *uid;			/* Univeral ID			*/
    uint16 language;
    uint16 admin;
    char *ctcp;
    int sqlid;			/* mysql id number */

    int cstats;          /* used for channelstats - dont modify it */
    char *sgroup;        /* contains the escaped name of the stats-group */

    ModuleData *moduleData;		/* defined for it, it should allow the module Add/Get */
    char *country_code;
    char *country_name;

    int isservice;
    
    struct u_chanlist {
        struct u_chanlist *next, *prev;
        Channel *chan;
        int16 status;    /* Associated flags; see CSTATUS_* below. */
    } *chans;            /* Channels user has joined */

};

struct cbmode_ {
    uint32 flag;     /* Long value that represents the mode */
    /* Function to associate a value with the mode */
    void (*setvalue) (Channel *chan, char *value);
};

struct cbmodeinfo_ {
	char mode;				/* The mode */
	uint32 flag;			/* Long value that represents the mode */
	/* Function to retrieve the value associated to the mode (optional) */
	char * (*getvalue) 		(Channel *chan);
};

struct cmmode_ {
	void (*addmask) (Channel *chan, char *mask);
	void (*delmask) (Channel *chan, char *mask);
};

struct cumode_ {
	int16 status;			/* CUS_* below */
	int16 flags;			/* CUF_* below */

	int (*is_valid) (User *user, Channel *chan, int servermode);
};

struct bandata_ {
    BanData *next, *prev;

    char *mask;				/* Since a nick is unsure and a User structure
    					   	   is unsafe */
    time_t last_use;		/* Since time is the only way to check
    					   	   whether it's still useful */
    int16 ttb;
};

#define CS_FANTASY        0x00000001
#define CS_NOTICE         0x00000002

struct chanstats_ {
    ChannelStats *next, *prev;
    char *name;
    uint32 flags;
};

struct statschan_ {
    StatsChannel *next, *prev;
    StatsChannel *nextsort, *prevsort;
    char name[CHANMAX];
    uint32 kickcount;
    uint32 joincounter;
    uint32 topic_count;
    uint16 usercount;
    uint16 maxusercount;
    time_t maxusertime;
    int in_use;
    ModuleData *moduleData;
};

struct channel_ {
    Channel *next, *prev;
    char name[CHANMAX];
    time_t creation_time;		/* When channel was created */
    char *topic;
    char topic_setter[NICKMAX];		/* Who set the topic */
    time_t topic_time;			/* When topic was set */
    uint32 mode;			/* Binary modes only */
    uint32 limit;			/* 0 if none */
    char *key;				/* NULL if none */
    char *redirect;			/* +L; NULL if none */
    char *flood;			/* +f; NULL if none */
    int32 bancount, bansize;
    char **bans;
    int32 exceptcount, exceptsize;
    char **excepts;
    int32 invitecount, invitesize;
    char **invite;
    struct c_userlist {
		struct c_userlist *next, *prev;
		User *user;
		UserData *ud;
    } *users;
	StatsChannel *stats;
    int cstats;          /* used for channelstats - dont modify it */
	int sqlid;

    BanData *bd;

    time_t server_modetime;		/* Time of last server MODE */
    time_t chanserv_modetime;	/* Time of last check_modes() */
    int16 server_modecount;		/* Number of server MODEs this second */
	ModuleData *moduleData;
};

/* Configuration structures */

#define MAXHOSTS 32

struct Conf_Admin
{
    char *hosts[MAXHOSTS+1];
    char *passwd;
    char *name;
    uint16 language;
    int   legal;
    int   admins;    /* number of opers currently using this */
    aAdmin *next;
};

struct Conf_Modules
{
    char *autoload[128];
    char *delayed[128];
};

/*************************************************************************/
/**
 * IRCD Protocol module support struct.
 * protocol modules register the command they want touse for function X with our set
 * functions, we then call the correct function for the anope_ commands.
 **/
typedef struct ircd_proto_ {
	void (*ircd_set_mod_current_buffer)(int ac, char **av);
    void (*ircd_cmd_nick)(char *nick, char *name, const char *modes);
	void (*ircd_cmd_bot_nick) (char *nick, char *user, char *host, char *real, char *modes);
    void (*ircd_cmd_mode)(char *source, char *dest, char *buf);
    void (*ircd_cmd_notice)(char *source, char *dest, char *buf);
    void (*ircd_cmd_privmsg)(char *source, char *dest, char *buf);
    void (*ircd_cmd_serv_notice)(char *source, char *dest, char *msg);
    void (*ircd_cmd_serv_privmsg)(char *source, char *dest, char *msg);
    void (*ircd_cmd_351)(char *source);
    void (*ircd_cmd_quit)(char *source, char *buf);
    void (*ircd_cmd_pong)(char *servname, char *who);
    void (*ircd_cmd_join)(char *user, char *channel, time_t chantime);
    void (*ircd_cmd_part)(char *nick, char *chan, char *buf);
    void (*ircd_cmd_global)(char *source, char *buf);
    void (*ircd_cmd_squit)(char *servname, char *message);
    void (*ircd_cmd_connect)(void);
    void (*ircd_cmd_ctcp)(char *source, char *dest, char *buf);
    void (*ircd_cmd_eob)(void);
    void (*ircd_cmd_version)(char *server);
    void (*ircd_cmd_stats)(char *sender, const char *letter, char *server);
    void (*ircd_cmd_motd)(char *sender, char *server);
    void (*ircd_cmd_ping)(char *server);
} IRCDProto;

typedef struct ircd_modes_ {
        int user_oper;
		uint32 ircd_var;
} IRCDModes;


#include "modules.h"
#include "language.h"
#include "extern.h"

/*************************************************************************/

#endif /* STATS_H */
