/* StatServ core functions
 *
 * (C) 2003-2005 Anope Team
 * Contact us at info@anope.org
 *
 * Please read COPYING and README for further details.
 *
 * Based on the original code of Epona by Lara.
 * Based on the original code of Services by Andy Church. 
 * 
 * $Id: ss_login.c 389 2005-07-05 22:45:48Z trystan $
 *
 */
/*************************************************************************/

#include "denora.h"

static int do_login(User * u);
static int do_logout(User * u);
int DenoraInit(int argc, char **argv);
void DenoraFini(void);

/**
 * Create the command, and tell anope about it.
 * @param argc Argument count
 * @param argv Argument list
 * @return MOD_CONT to allow the module, MOD_STOP to stop it
 **/
int DenoraInit(int argc, char **argv)
{
    Command *c;

    if (denora->debug >= 2) {
        protocol_debug(NULL, argc, argv);
    }
    moduleAddAuthor("Denora");
    moduleAddVersion("$Id: ss_login.c 389 2005-07-05 22:45:48Z trystan $");
    moduleSetType(CORE);

    c = createCommand("LOGIN",     do_login,      is_oper,             STAT_HELP_LOGIN,  -1,-1,-1);
    moduleAddCommand(STATSERV, c, MOD_UNIQUE);

    c = createCommand("LOGOUT",    do_logout,     NULL,                STAT_HELP_LOGOUT, -1,-1,-1);
    moduleAddCommand(STATSERV, c, MOD_UNIQUE);

    return MOD_CONT;
}

/**
 * Unload the module
 **/
void DenoraFini(void)
{

}

/*************************************************************************/

/* LOGIN command */
static int do_login(User * u)
{
    aAdmin *aadmin;
    char *temp = moduleGetLastBuffer();
    char *name = myStrGetToken(temp, ' ', 0);
    char *passwd = myStrGetToken(temp, ' ', 1);

    /* Required params */
    if (!name || !passwd) {
        syntax_error(s_StatServ, u, "LOGIN", STAT_LOGIN_SYNTAX);
        denora_free(name);
        denora_free(passwd);
        return MOD_CONT;
    }

    alog(LOG_NORMAL, "%s: %s: LOGIN %s ****", s_StatServ, u->nick, name);

    /* Trying to login again? */
    if (u->admin) {
        notice_lang(s_StatServ, u, STAT_ADMIN_ALREADY_LOGGED);
        denora_free(name);
        denora_free(passwd);
        return MOD_CONT;
    }

    aadmin = find_admin(name, u);
    if (!aadmin) {
        u->admin = 0;
    } else {
        if (!stricmp(passwd, aadmin->passwd)) {
            u->admin = 1;
            /* Use the language set in denora.conf */
            if (aadmin->language) {
                u->language = aadmin->language;
            }
        }
    }

    /* The user has logged in */
    if (u->admin) {
        denora_cmd_global(s_StatServ, langstring(NOW_STATS_ADMIN),
                          u->nick, name);
        notice_lang(s_StatServ, u, STAT_ADMIN_LOGGED_IN);
    } else {
        denora_cmd_global(s_StatServ, langstring(STATS_AUTH_FAILED),
                          u->nick);
        notice_lang(s_StatServ, u, ACCESS_DENIED);
    }

    denora_free(name);
    denora_free(passwd);

    return MOD_CONT;
}

/*************************************************************************/

/* LOGOUT command */
static int do_logout(User * u)
{
    if (u->admin) {
        u->admin = 0;
    } else {
        return MOD_CONT;
    }

    if (!u->admin) {
        denora_cmd_global(s_StatServ, langstring(STATS_ADMIN_LOGOUT),
                          u->nick);
        notice_lang(s_StatServ, u, STAT_ADMIN_LOGGED_OUT);
    }
    return MOD_CONT;
}

/*************************************************************************/

