
/* signals
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: signals.c 403 2005-07-10 06:09:11Z trystan $
 *
 */

#include "denora.h"

VOIDSIG signal_segfault(int signum);
VOIDSIG signal_restart(int signum);
VOIDSIG signal_rehash(int signum);
VOIDSIG signal_int(int signum);
VOIDSIG signal_pipe(int signum);
VOIDSIG signal_die(int signum);

#ifndef strsignal
char *strsignal(int sig);
#endif

/*************************************************************************/

char segv_location[SEGV_LOCATION_BUFSIZE];

#ifndef _WIN32
void signal_init()
{
#ifdef	POSIX_SIGNALS
    struct sigaction act;
    act.sa_handler = SIG_IGN;
    act.sa_flags = 0;

# ifdef	SIGWINCH
    act.sa_handler = SIG_IGN;
    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGWINCH);
    (void) sigaction(SIGWINCH, &act, NULL);
# endif

    act.sa_handler = signal_pipe;
    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGPIPE);
    (void) sigaction(SIGPIPE, &act, NULL);

    act.sa_handler = sighandler;
    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGALRM);
    (void) sigaction(SIGALRM, &act, NULL);

    act.sa_handler = signal_rehash;
    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGHUP);
    (void) sigaction(SIGHUP, &act, NULL);

    act.sa_handler = signal_restart;
    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGUSR2);
    (void) sigaction(SIGUSR2, &act, NULL);

    /* Only die with SIGINT if we are in nofork mode */
    if (denora->nofork) {
        act.sa_handler = signal_int;
        (void) sigemptyset(&act.sa_mask);
        (void) sigaddset(&act.sa_mask, SIGINT);
        (void) sigaction(SIGINT, &act, NULL);
    }

    act.sa_handler = signal_die;
    (void) sigaddset(&act.sa_mask, SIGTERM);
    (void) sigaction(SIGTERM, &act, NULL);
    if (!DumpCore) {
        act.sa_handler = signal_segfault;
        (void) sigemptyset(&act.sa_mask);
        (void) sigaddset(&act.sa_mask, SIGSEGV);
        (void) sigaction(SIGSEGV, &act, NULL);
    } else {
        (void) signal(SIGSEGV, SIG_DFL);
    }
#else
# ifndef	HAVE_RELIABLE_SIGNALS
    (void) signal(SIGPIPE, sighandler);
#  ifdef	SIGWINCH
    (void) signal(SIGWINCH, sighandler);
#  endif
# else
#  ifdef	SIGWINCH
    (void) signal(SIGWINCH, SIG_IGN);
#  endif
    (void) signal(SIGPIPE, SIG_IGN);
# endif
    (void) signal(SIGALRM, sighandler);
    (void) signal(SIGHUP, signal_rehash);
    (void) signal(SIGTERM, signal_die);
    (void) signal(SIGUSR2, signal_restart);
    (void) signal(SIGPIPE, signal_pipe);

    if (denora->nofork) {
        (void) signal(SIGINT, signal_int);
    }
    if (!DumpCore) {
        (void) signal(SIGSEGV, signal_segfault);
    } else {
        (void) signal(SIGSEGV, SIG_DFL);
    }
#endif

#ifdef SIGIOT
    (void) signal(SIGIOT, sighandler);
#endif
    (void) signal(SIGFPE, sighandler);
    (void) signal(SIGQUIT, sighandler);
    if (!DumpCore) {
        (void) signal(SIGBUS, sighandler);
        (void) signal(SIGILL, sighandler);
        (void) signal(SIGTRAP, sighandler);
    } else {
        (void) signal(SIGBUS, SIG_DFL);
        (void) signal(SIGILL, SIG_DFL);
        (void) signal(SIGTRAP, SIG_DFL);
    }

#if !defined(USE_THREADS) || !defined(LINUX20)
    (void) signal(SIGUSR1, sighandler); /* This is our "out-of-memory" panic switch */
#endif

}
#else
void signal_init()
{
    if (!DumpCore) {
        (void) signal(SIGSEGV, signal_segfault);
    } else {
        (void) signal(SIGSEGV, SIG_DFL);
    }
}
#endif

/*************************************************************************/

VOIDSIG signal_pipe(int signum)
{
#ifdef SIGPIPE
    if (signum == SIGPIPE) {
#endif
     send_event(EVENT_SIGNAL, 2, "SIGPIPE", "Received SIGPIPE: Broken pipe");
     alog(LOG_NORMAL, "Received SIGPIPE: Broken pipe");
     alog(LOG_NORMAL, langstr(ALOG_SIGPIPE_WARNING));
     do_backtrace(1);
#ifdef SIGPIPE
    }
#endif
}

/*************************************************************************/

VOIDSIG signal_segfault(int signum)
{
    if (signum == SIGSEGV) {
     send_event(EVENT_SIGNAL, 2, "SIGSEGV", "Received SIGSEGV: Segfault");
     alog(LOG_NORMAL, "Received SIGSEGV: Segfault");
     do_backtrace(1);
     exit(0);
    }
}

/*************************************************************************/

VOIDSIG signal_restart(int signum)
{
#ifdef SIGUSR2
    if (signum == SIGUSR2) {
#endif
     alog(LOG_NORMAL, "Received SIGUSR2, restarting.");
     if (!denora->qmsg) {
        denora->qmsg = sstrdup("Restarting on SIGUSR2");
     }
     send_event(EVENT_SIGNAL, 2, "SIGUSR2", denora->qmsg);
#ifdef STATS_BIN
     denora_restart();
#else
     denora->qmsg = sstrdup(langstr(ALOG_NO_STATS_BIN));
#endif
     exit(-1);
#ifdef SIGUSR2
    }
#endif
}

/*************************************************************************/

VOIDSIG signal_rehash(int signum)
{
#ifdef	POSIX_SIGNALS
    struct sigaction act;
#endif
#ifdef SIGHUP
    if (signum == SIGHUP) {
#endif
     alog(LOG_NORMAL,
         "Received SIGHUP: Saving Databases & Rehash Configuration");
     if (initconf(denora->config, 1) == -1) {
        denora->qmsg = calloc(50, 1);
        if (!denora->qmsg) {
            denora->qmsg = sstrdup(langstr(ALOG_ERR_REHASH_NO_MEM));
        } else {
            sprintf(denora->qmsg, langstr(ALOG_ERR_REHASH));
        }
        denora->quitting = 1;
        send_event(EVENT_SIGNAL, 2, "SIGHUP", denora->qmsg);
     } else {
        merge_confs();
     }
#ifdef	POSIX_SIGNALS
    act.sa_handler = signal_rehash;
    act.sa_flags = 0;

    (void) sigemptyset(&act.sa_mask);
    (void) sigaddset(&act.sa_mask, SIGHUP);
    (void) sigaction(SIGHUP, &act, NULL);
#else
# ifndef _WIN32
    (void) signal(SIGHUP, signal_rehash);       /* sysV -argv */
# endif
#endif
#ifdef SIGHUP
    }
#endif
}

/*************************************************************************/

VOIDSIG signal_die(int signum)
{
    if (signum == SIGTERM) {
     alog(LOG_NORMAL, "Received SIGTERM, exiting.");
     denora->qmsg = sstrdup("Shutting down on SIGTERM");
     send_event(EVENT_SIGNAL, 2, "SIGTERM", denora->qmsg);
     denora_shutdown();
     exit(-1);
    }
}

/*************************************************************************/

VOIDSIG signal_int(int signum)
{
    if (signum == SIGINT) {
     alog(LOG_NORMAL, "Received SIGINT, exiting.");
     denora->qmsg = sstrdup("Shutting down on SIGINT");
     send_event(EVENT_SIGNAL, 2, "SIGINT", denora->qmsg);
     denora_shutdown();
     exit(-1);
    }
}

/*************************************************************************/

/* If we get a weird signal, come here. */

VOIDSIG sighandler(int signum)
{
#ifndef _WIN32
    if (started) {
        if (signum == SIGQUIT) {
            /* nothing -- terminate below */
        } else if (!waiting) {
            alog(LOG_NORMAL, "PANIC! buffer = %s", inbuf);
            /* Cut off if this would make IRC command >510 characters. */
            if (strlen(inbuf) > 448) {
                inbuf[446] = '>';
                inbuf[447] = '>';
                inbuf[448] = 0;
            }
            denora_cmd_global(NULL, "PANIC! buffer = %s\r\n", inbuf);
        } else if (waiting < 0) {
            /* This is static on the off-chance we run low on stack */
            static char buf[BUFSIZE];
            switch (waiting) {
            case -1:
                snprintf(buf, sizeof(buf), "in timed_update");
                break;
            default:
                snprintf(buf, sizeof(buf), "waiting=%d", waiting);
            }
#ifdef HAVE_STRSIGNAL
            denora_cmd_global(NULL, "PANIC! %s (%s)", buf,
                              strsignal(signum));
#endif
            alog(LOG_NORMAL, "PANIC! %s (%s)", buf, strsignal(signum));
           
        }
    }
#endif
    if (
#if !defined(USE_THREADS) || !defined(LINUX20)
#ifndef _WIN32
           signum == SIGUSR1 ||
#endif
#endif
           !(denora->qmsg = calloc(BUFSIZE, 1))) {
        denora->qmsg = sstrdup("Out of memory!");
    } else {
        snprintf(denora->qmsg, BUFSIZE, "Stats terminating: %s",
                 strsignal(signum));
    }
    send_event(EVENT_SIGNAL, 2, "unknown", denora->qmsg);
    if (started) {
        denora_shutdown();
        exit(0);
    } else {
        if (isatty(2)) {
            fprintf(stderr, "%s\n", denora->qmsg);
        } else {
            alog(LOG_NORMAL, "%s", denora->qmsg);
        }
        exit(1);
    }
    return;
}

/*************************************************************************/

void do_backtrace(int header)
{
#ifndef _WIN32
#ifdef HAVE_BACKTRACE
    void *array[50];
    size_t size;
    char **strings;
    int i;

    if (header) {
        alog(LOG_NORMAL, "Backtrace: Segmentation fault detected");
        alog(LOG_NORMAL, langstr(BACKTRACE_LINES));
        if (segv_location) {
            alog(LOG_NORMAL, "Backtrace: %s", segv_location);
        }
        alog(LOG_NORMAL,
             "Backtrace: Denora version %s build #%s, compiled %s %s",
             denora->version, denora->build, denora->date, denora->time);
    } else {
        if (segv_location) {
            alog(LOG_NORMAL, "Execution trace: %s", segv_location);
        }
        alog(LOG_NORMAL, "Execution trace: in progress");
    }
    size = backtrace(array, 10);
    strings = backtrace_symbols(array, size);
    for (i = 1; i < (int) size; i++) {
        alog(LOG_NORMAL, "%s(%d): %s",
             (header ? "Backtrace" : "Execution Trace"), i, strings[i]);
    }
    denora_free(strings);
    alog(LOG_NORMAL, "%s: complete",
         (header ? "Backtrace" : "Execution Trace"));
#else
    if (segv_location) {
        alog(LOG_NORMAL,"Backtrace: %s", segv_location);
    }
    if (header) {
        alog(LOG_NORMAL, langstr(BACKTRACE_NOT_HERE));
    }
#endif
#else
    if (segv_location) {
      alog(LOG_NORMAL, "Backtrace: %s", segv_location);
    }
    alog(LOG_NORMAL, "Backtrace not supported on win32");
#endif
}

/*************************************************************************/



